const fs = require('fs');
const path = require('path');
const homeDir = require('homedir');

class Keys {

    constructor(fileName) {
        this.keys = {};
        this.keyFile = path.join(homeDir(), fileName);
    }

    set(domain, secret) {
        return new Promise((resolve, reject) => {
            this.loadKeyFile();
            this.keys[domain] = secret;
            this.saveKeyFile();
            resolve();
        });
    }

    get(domain) {
        return new Promise((resolve, reject) => {
            this.loadKeyFile();
            if (this.keys[domain]) {
                resolve(this.keys[domain]);
            } else {
                reject('Domain ' + domain + ' not registered! Please run atajo register add <domain> <secret>');
            }
        })

    }

    list() {
        return new Promise((resolve, reject) => {
            this.loadKeyFile();
            resolve(this.keys);
        })
    }

    del(domain) {
        return new Promise((resolve) => {
            this.loadKeyFile();
            delete this.keys[domain];
            this.saveKeyFile();
            resolve();
        })
    }

    loadKeyFile() {
        try {
            this.keys = JSON.parse(fs.readFileSync(this.keyFile, 'utf8'));
        } catch (e) {
            this.keys = {};
        }
        // console.log("loadKeyFile => ", this.keys);
    }

    saveKeyFile() {
        // console.log("saveKeyFile => ", this.keys);
        fs.writeFileSync(this.keyFile, JSON.stringify(this.keys));
    }





}

module.exports = Keys;