const fs = require('fs');
const ncp = require('ncp');
const path = require('path');
const main = require('caporal')('atajo lambda');
const cp = require('child_process');
const Keys = require('../keys');
const Controller = require('../io/atajo.controller'); 
const cpuLen = require('os')
    .cpus()
    .length;

const ioBuffer = [];

main
    .version(version)
    .command('test')
    .argument('<domain>', 'Domain to test - i.e. run locally in your terminal', /^([0-9]|[a-z]|-|_)+([0-9a-z]+)$/i)
    .description('Starts Lambdas in the current directory on this machine')
    .help('Starts Lambdas in the current directory on this machine')
    .action(function (args, options) {

        const lambdaPath = path.join(process.cwd(), 'lambdas');
        if (!fs.existsSync(lambdaPath)) {
            console.error("./lambdas not found in current directory. Please run  atajo lambda create  to create sample lambdas");
            return;
        }

        const domain = args.domain;
        let keys = new Keys('.atajo.keys');

        keys
            .get(domain)
            .then(secret => {

                console.log();
                console.log();
                console.log("-----------------------------------------------------------------");
                console.log("   Starting lambdas (in local mode) for domain : " + domain);
                console.log("-----------------------------------------------------------------");
                console.log();
                console.log();

                const controller = new Controller(domain, secret); 
                controller.standalone(); 
                
            })
            .catch(e => {
                console.error(e);
            });

    })
    .command('run')
    .argument('<domain>', 'Domain to run in production mode', /^([0-9]|[a-z]|-|_)+([0-9a-z]+)$/i)
    .description('Runs a Lambda controller in production mode (Using all available resources)')
    .help('Builds a Lambda container and runs it on this machine (Using all available resou' +
            'rces)')
    .action(function (args, options) {

        const lambdaPath = path.join(process.cwd(), 'lambdas');
        if (!fs.existsSync(lambdaPath)) {
            console.error("./lambdas not found in current directory. Please run  atajo lambda create  to cr" +
                    "eate sample lambdas");
            return;
        }

        const domain = args.domain;
        const environment = args.environment || 'dev';
        let keys = new Keys('.atajo.keys');

        const ioPath = path.join(__dirname, '../', 'io', 'start.js');

        keys
            .get(domain)
            .then(secret => {

                console.log();
                console.log();
                console.log("-----------------------------------------------------------------");
                console.log("   Starting production lambdas for domain : " + domain);
                console.log("-----------------------------------------------------------------");
                console.log();
                console.log();


                const controller = new Controller(environment, domain, secret); 
                controller.cluster(); 

            })
            .catch(e => {
                console.error(e);
            });

    })
    .command('container')
    .argument('<domain>', 'Domain to build container for and run', /^([0-9]|[a-z]|-|_)+([0-9a-z]+)$/i)
    .description('Builds a Lambda container and runs it on this machine - This assumes you have do' +
            'cker installed.')
    .help('Builds a Lambda container and runs it on this machine - This assumes you have do' +
            'cker installed.')
    .action(function (args, options) {

        const lambdaPath = path.join(process.cwd(), 'lambdas');
        if (!fs.existsSync(lambdaPath)) {
            console.error("./lambdas not found in current directory. Please run  atajo lambda create  to cr" +
                    "eate sample lambdas");
            return;
        }

        const domain = args.domain;
        const environment = args.environment || 'dev';
        let keys = new Keys('.atajo.keys');

        const ioPath = path.join(__dirname, '../', 'io', 'start.js');

        keys
            .get(domain)
            .then(secret => {

                console.log();
                console.log();
                console.log("-----------------------------------------------------------------");
                console.log("   Starting containerized lambdas for domain : " + domain);
                console.log("-----------------------------------------------------------------");
                console.log();
                console.log();

                //COPY DOCKERFILE
                fs.copyFileSync(path.join(__dirname, 'compose', 'Dockerfile'), path.join(process.cwd(), 'DockerFile'));

                //PROCESS COMPOSE
                let compose = fs.readFileSync(path.join(__dirname, 'compose', 'docker-compose-aio.yml'), 'utf8');
                compose = compose.replace(/{{domain}}/g, domain);
                compose = compose.replace(/{{secret}}/g, secret);
                fs.writeFileSync(path.join(process.cwd(), 'docker-compose-aio.yml'), compose);

                //CREATE ENV
                const envPath = path.join(process.cwd(), 'domain.env')
                const DOMAIN_ENV = "DOMAIN=" + domain + "\r\nSECRET=" + secret + "\r\n";
                fs.writeFileSync(envPath, DOMAIN_ENV);

                console.log("DONE");

            })
            .catch(e => {
                console.error(e);
            });

    })
    .command('create')
    .description('Creates a lambda directory containing sample lambdas in the current directory')
    .help('Creates a lambda directory containing sample lambdas in the current directory')
    .action(function (args, options) {

        let lambdaDirectory = path.join(process.cwd(), 'lambdas');

        if (fs.existsSync(lambdaDirectory)) {

            console.warn("./lambdas - Directory already exists!");
            return;

        }

        ncp(path.join(__dirname, 'lambdas'), lambdaDirectory, function (err) {
            if (err) {
                return console.error(err);
            }
            console.log('./lambdas created. You can run    atajo lambda run <domain>    to run your lambd' +
                    'as in developer mode');
        });

    })
    .command('deploy')
    .description('Deploys your Lambdas in a scalable cloud container to the cloud provider of choi' +
            'ce (COMING SOON)')
    .help('Deploys your Lambdas in a scalable cloud container to the cloud provider of choi' +
            'ce (COMING SOON)')
    .action(function (args, options) {

        console.log("COMING SOON. FOR NOW PLEASE ASK werner.venter@britehouse.co.za TO DEPLOY LAMBDAS" +
                " FOR YOU");

    })

process
    .argv
    .splice(2, 1);
process.argv[1] = 'atajo lambda';
main.parse(process.argv);